using System;
using System.IO;
using System.Collections;
using System.Text.RegularExpressions;

namespace VCCCompiler
{
        /// <summary>
        /// Zusammenfassung fr MyCompiler.
        /// </summary>
        class MyCompiler
        {
                YYARec[] yya;
                YYARec[] yyg;
                YYRRec[] yyr;
                int[] yyd;
                int[] yyal;
                int[] yyah;
                int[] yygl;
                int[] yygh;

                int yyn = 0;
                int yystate = 0;
                int yychar = -1;
                int yynerrs = 0;
                int yyerrflag = 0;
                int yysp = 0;
                int yymaxdepth = 1024;
                int yyflag = 0;
                int yyfnone   = 0;
                int[] yys = new int[1024];
                string[] yyv = new string[1024];

                string yyval = "";

                FileStream OutputStream;
                StreamWriter Output;

                class YYARec
                {
                        public int sym;
                        public int act;
                        public YYARec (int s, int a){ sym = s; act = a; }
                }

                class YYRRec
                {
                        public int len;
                        public int sym;
                        public YYRRec (int l, int s){ sym = s; len = l; }
                }

                ////////////////////////////////////////////////////////////////
                /// Constant values / tokens
                ////////////////////////////////////////////////////////////////
                int t_zahl = 257;
                int t_KlammerAuf = 258;
                int t_KlammerZu = 259;
                int t_VW = 260;
                int t_RE = 261;
                int t_WH = 262;
                int t_FARBE = 263;
                int t_STIFT = 264;
                int t_farbwert = 265;
                int t_ignore = 256;
///////////////////////////////////////////////////////////
/// Global settings: 
///////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////


                /// <summary>
                /// Der Haupteinstiegspunkt fr die Anwendung.
                /// </summary>
                [STAThread]
                static int Main(string[] args)
                {

                        bool ShowTokens = false;
                        string InputFilename = "";
                        string OutputFilename = "";

                        foreach (string s in args)
                        {
                                if (s.ToLower() == "-t")
                                {
                                        ShowTokens = true;
                                }
                                else
                                {
                                        if (InputFilename == "")  InputFilename = s; else
                                        if (OutputFilename == "") OutputFilename = s; else
                                        {
                                                Console.WriteLine("Too many arguments!");
                                                return 1;
                                        }
                                }
                        }
                        if (InputFilename == "")
                        {
                                System.Console.WriteLine("You need to specify input and outputfile: compiler.exe input.txt output.txt");
                                return 1;
                        }

                        StreamReader in_s = File.OpenText(InputFilename);
                        string inputstream = in_s.ReadToEnd();
                        in_s.Close();

                        ////////////////////////////////////////////////////////////////
                        /// Compiler Code:
                        ////////////////////////////////////////////////////////////////
                        MyCompiler compiler = new MyCompiler();
                        compiler.Output = null;
                        if (OutputFilename != "")
                        {
                                File.Delete(OutputFilename);
                                compiler.OutputStream = File.OpenWrite(OutputFilename);
                                compiler.Output = new StreamWriter(compiler.OutputStream);
                        }
                        else
                        {
                                compiler.Output = new StreamWriter(Console.OpenStandardOutput());

                        }

                        if (!compiler.Scanner(inputstream)) return 1;
                        if (ShowTokens)
                        {
                                foreach (AToken t in compiler.TokenList)
                                {
                                        Console.WriteLine("TokenID: "+t.token+"  =  "+t.val);
                                }
                        }
                        compiler.InitTables();
                        if (!compiler.yyparse()) return 1;

                        if (compiler.Output != null) compiler.Output.Close();
			return 0;
                }
                public void yyaction (int yyruleno)
                {
                        switch (yyruleno)
                        {
                                ////////////////////////////////////////////////////////////////
                                /// YYAction code:
                                ////////////////////////////////////////////////////////////////
							case    1 : 
         String s = "";
         s += "%!PS-Adobe-2.0 \r\n";
         s += "/orient 0 def /xpos 0 def /ypos 0 def \r\n";
         s += "0 0 0 setrgbcolor\r\n"; // Stiftfarbe
         s += "/goto { /ypos exch def /xpos exch def xpos ypos moveto} def\r\n";
         s += "/turn { /orient exch orient add def} def \r\n";
         s += "/draw { /len exch def newpath xpos ypos moveto \r\n";
         s += "   /xpos xpos orient sin len mul add def \r\n";
         s += "   /ypos ypos orient cos len mul add def \r\n";
         s += "   xpos ypos lineto stroke \r\n";
         s += "} def \r\n";
         s += "300 400 goto\r\n"; // in Blattmitte beginnen
         
         s += yyv[yysp-0]+"\r\n";      
         
         Output.WriteLine(s);
         
       break;
							case    2 : 
         yyval = yyv[yysp-0];
         
       break;
							case    3 : 
         yyval = yyv[yysp-1] + yyv[yysp-0];
         
       break;
							case    4 : 
         yyval = yyv[yysp-0]+" draw ";
         
       break;
							case    5 : 
         yyval = yyv[yysp-0]+" turn ";
         
       break;
							case    6 : 
         yyval = "";
         for (int i = 0; i < Int32.Parse(yyv[yysp-3]); i++) yyval += yyv[yysp-1];
         
       break;
							case    7 : 
         if (yyv[yysp-0] == "blau")    yyval = "0 0 255 setrgbcolor ";
         if (yyv[yysp-0] == "rot")     yyval = "255 0 0 setrgbcolor ";
         if (yyv[yysp-0] == "gruen")   yyval = "0 255 0 setrgbcolor ";
         if (yyv[yysp-0] == "gelb")    yyval = "255 255 0 setrgbcolor ";
         if (yyv[yysp-0] == "schwarz") yyval = "0 0 0 setrgbcolor ";
         
       break;
							case    8 : 
         yyval = yyv[yysp-0]+" setlinewidth ";
         
       break;
                                default : return;
                        }
                }

                public void InitTables()
                {
                        ////////////////////////////////////////////////////////////////
                        /// Init Table code:
                        ////////////////////////////////////////////////////////////////

					int yynacts   = 25;
					int yyngotos  = 7;
					int yynstates = 18;
					int yynrules  = 8;
					yya = new YYARec[yynacts+1];  int yyac = 1;
					yyg = new YYARec[yyngotos+1]; int yygc = 1;
					yyr = new YYRRec[yynrules+1]; int yyrc = 1;

					yya[yyac] = new YYARec(260,4);yyac++; 
					yya[yyac] = new YYARec(261,5);yyac++; 
					yya[yyac] = new YYARec(262,6);yyac++; 
					yya[yyac] = new YYARec(263,7);yyac++; 
					yya[yyac] = new YYARec(264,8);yyac++; 
					yya[yyac] = new YYARec(260,4);yyac++; 
					yya[yyac] = new YYARec(261,5);yyac++; 
					yya[yyac] = new YYARec(262,6);yyac++; 
					yya[yyac] = new YYARec(263,7);yyac++; 
					yya[yyac] = new YYARec(264,8);yyac++; 
					yya[yyac] = new YYARec(0,-2 );yyac++; 
					yya[yyac] = new YYARec(259,-2 );yyac++; 
					yya[yyac] = new YYARec(0,0);yyac++; 
					yya[yyac] = new YYARec(257,10);yyac++; 
					yya[yyac] = new YYARec(257,11);yyac++; 
					yya[yyac] = new YYARec(257,12);yyac++; 
					yya[yyac] = new YYARec(265,13);yyac++; 
					yya[yyac] = new YYARec(257,14);yyac++; 
					yya[yyac] = new YYARec(258,15);yyac++; 
					yya[yyac] = new YYARec(260,4);yyac++; 
					yya[yyac] = new YYARec(261,5);yyac++; 
					yya[yyac] = new YYARec(262,6);yyac++; 
					yya[yyac] = new YYARec(263,7);yyac++; 
					yya[yyac] = new YYARec(264,8);yyac++; 
					yya[yyac] = new YYARec(259,17);yyac++;

					yyg[yygc] = new YYARec(-4,1);yygc++; 
					yyg[yygc] = new YYARec(-3,2);yygc++; 
					yyg[yygc] = new YYARec(-2,3);yygc++; 
					yyg[yygc] = new YYARec(-4,1);yygc++; 
					yyg[yygc] = new YYARec(-3,9);yygc++; 
					yyg[yygc] = new YYARec(-4,1);yygc++; 
					yyg[yygc] = new YYARec(-3,16);yygc++;

					yyd = new int[yynstates];
					yyd[0] = 0;  
					yyd[1] = 0;  
					yyd[2] = -1;  
					yyd[3] = 0;  
					yyd[4] = 0;  
					yyd[5] = 0;  
					yyd[6] = 0;  
					yyd[7] = 0;  
					yyd[8] = 0;  
					yyd[9] = -3;  
					yyd[10] = -4;  
					yyd[11] = -5;  
					yyd[12] = 0;  
					yyd[13] = -7;  
					yyd[14] = -8;  
					yyd[15] = 0;  
					yyd[16] = 0;  
					yyd[17] = -6; 

					yyal = new int[yynstates];
					yyal[0] = 1;  
					yyal[1] = 6;  
					yyal[2] = 13;  
					yyal[3] = 13;  
					yyal[4] = 14;  
					yyal[5] = 15;  
					yyal[6] = 16;  
					yyal[7] = 17;  
					yyal[8] = 18;  
					yyal[9] = 19;  
					yyal[10] = 19;  
					yyal[11] = 19;  
					yyal[12] = 19;  
					yyal[13] = 20;  
					yyal[14] = 20;  
					yyal[15] = 20;  
					yyal[16] = 25;  
					yyal[17] = 26; 

					yyah = new int[yynstates];
					yyah[0] = 5;  
					yyah[1] = 12;  
					yyah[2] = 12;  
					yyah[3] = 13;  
					yyah[4] = 14;  
					yyah[5] = 15;  
					yyah[6] = 16;  
					yyah[7] = 17;  
					yyah[8] = 18;  
					yyah[9] = 18;  
					yyah[10] = 18;  
					yyah[11] = 18;  
					yyah[12] = 19;  
					yyah[13] = 19;  
					yyah[14] = 19;  
					yyah[15] = 24;  
					yyah[16] = 25;  
					yyah[17] = 25; 

					yygl = new int[yynstates];
					yygl[0] = 1;  
					yygl[1] = 4;  
					yygl[2] = 6;  
					yygl[3] = 6;  
					yygl[4] = 6;  
					yygl[5] = 6;  
					yygl[6] = 6;  
					yygl[7] = 6;  
					yygl[8] = 6;  
					yygl[9] = 6;  
					yygl[10] = 6;  
					yygl[11] = 6;  
					yygl[12] = 6;  
					yygl[13] = 6;  
					yygl[14] = 6;  
					yygl[15] = 6;  
					yygl[16] = 8;  
					yygl[17] = 8; 

					yygh = new int[yynstates];
					yygh[0] = 3;  
					yygh[1] = 5;  
					yygh[2] = 5;  
					yygh[3] = 5;  
					yygh[4] = 5;  
					yygh[5] = 5;  
					yygh[6] = 5;  
					yygh[7] = 5;  
					yygh[8] = 5;  
					yygh[9] = 5;  
					yygh[10] = 5;  
					yygh[11] = 5;  
					yygh[12] = 5;  
					yygh[13] = 5;  
					yygh[14] = 5;  
					yygh[15] = 7;  
					yygh[16] = 7;  
					yygh[17] = 7; 

					yyr[yyrc] = new YYRRec(1,-2);yyrc++; 
					yyr[yyrc] = new YYRRec(1,-3);yyrc++; 
					yyr[yyrc] = new YYRRec(2,-3);yyrc++; 
					yyr[yyrc] = new YYRRec(2,-4);yyrc++; 
					yyr[yyrc] = new YYRRec(2,-4);yyrc++; 
					yyr[yyrc] = new YYRRec(5,-4);yyrc++; 
					yyr[yyrc] = new YYRRec(2,-4);yyrc++; 
					yyr[yyrc] = new YYRRec(2,-4);yyrc++;
                }

                public bool yyact(int state, int sym , ref int act)
                {
                        int k = yyal[state];
                        while ( k <= yyah[state] && yya[k].sym != sym) k++;
                        if (k > yyah[state]) return false;
                        act = yya[k].act;
                        return true;
                }
                public bool yygoto(int state, int sym , ref int nstate)
                {
                        int k = yygl[state];
                        while ( k <= yygh[state] && yyg[k].sym != sym) k++;
                        if (k > yygh[state]) return false;
                        nstate = yyg[k].act;
                        return true;
                }

                public void yyerror (string s)
                {
                        System.Console.Write(s);
                }

                int yylexpos = -1;
                string yylval = "";

                public int yylex ()
                {
                        yylexpos++;
                        if(yylexpos >= TokenList.Count)
                        {
                                yylval = "";
                                return 0;
                        }
                        else
                        {
                                yylval = ((AToken)TokenList[yylexpos]).val;
                                return ((AToken)TokenList[yylexpos]).token;
                        }
                }

                public bool yyparse ()
                {

                        parse:

                                yysp++;
                        if (yysp>yymaxdepth)
                        {
                                yyerror("yyparse stack overflow");
                                goto abort;
                        }

                        yys[yysp] = yystate;
                        yyv[yysp] = yyval;

                        next:

                                if (yyd[yystate]==0 && yychar==-1)
                                {
                                        yychar = yylex();
                                        if (yychar<0) yychar = 0;
                                }

                        yyn = yyd[yystate];
                        if (yyn != 0) goto reduce;


                        if (! yyact(yystate, yychar, ref yyn)) goto error;
                        else if (yyn>0) goto shift;
                        else if (yyn<0) goto reduce;
                        else            goto accept;

                        error:

                                if (yyerrflag==0) yyerror("syntax error");

                        errlab:

                                if (yyerrflag==0) yynerrs++;

                        if (yyerrflag<=2)
                        {
                                yyerrflag = 3;
                                while (yysp>0 && !(yyact(yys[yysp], 255, ref yyn) && yyn > 0)) yysp--;

                                if (yysp==0) goto abort;
                                yystate = yyn;
                                goto parse;
                        }
                        else
                        {
                                if (yychar==0) goto abort;
                                yychar = -1; goto next;
                        }

                        shift:

                        yystate = yyn;
                        yychar = -1;
                        yyval = yylval;
                        if (yyerrflag>0) yyerrflag--;
                        goto parse;

                        reduce:

                        yyflag = yyfnone;
                        yyaction(-yyn);
                        yysp -= yyr[-yyn].len;

                        if (yygoto(yys[yysp], yyr[-yyn].sym, ref yyn)) yystate = yyn;

                        switch (yyflag)
                        {
                                case 1 : goto accept;
                                case 2 : goto abort;
                                case 3 : goto errlab;
                        }

                        goto parse;

                        accept:

                                return true;

                        abort:

                                return false;
                }
		////////////////////////////////////////////////////////////////
		/// Scanner
		////////////////////////////////////////////////////////////////

		public class AToken
		{
			public int token;
			public string val;
		}

		ArrayList TokenList = new ArrayList();

		public bool Scanner (string Input)
		{
		        if (Input.Length == 0) return true;
			TokenList = new ArrayList();
			while (1==1)
			{
				AToken lasttoken = FindToken(Input);
				if (lasttoken.token == 0) break;
				if (lasttoken.token != t_ignore) TokenList.Add(lasttoken);
				if (Input.Length > lasttoken.val.Length)
				Input = Input.Substring(lasttoken.val.Length); else return true;
			}
                      System.Console.WriteLine(Input);
			System.Console.WriteLine();
			System.Console.WriteLine("No matching token found!");
			return false;
		}
		public AToken FindToken (string Rest)
		{
			ArrayList Results  = new ArrayList();
			ArrayList ResultsV = new ArrayList();
                      try{

			if (Regex.IsMatch(Rest,"^([0-9]+)",RegexOptions.IgnoreCase)){
				Results.Add (t_zahl);
				ResultsV.Add(Regex.Match(Rest,"^([0-9]+)",RegexOptions.IgnoreCase).Value);}

			if (Regex.IsMatch(Rest,"^(\\[)",RegexOptions.IgnoreCase)){
				Results.Add (t_KlammerAuf);
				ResultsV.Add(Regex.Match(Rest,"^(\\[)",RegexOptions.IgnoreCase).Value);}

			if (Regex.IsMatch(Rest,"^(\\])",RegexOptions.IgnoreCase)){
				Results.Add (t_KlammerZu);
				ResultsV.Add(Regex.Match(Rest,"^(\\])",RegexOptions.IgnoreCase).Value);}

			if (Regex.IsMatch(Rest,"^(VW)",RegexOptions.IgnoreCase)){
				Results.Add (t_VW);
				ResultsV.Add(Regex.Match(Rest,"^(VW)",RegexOptions.IgnoreCase).Value);}

			if (Regex.IsMatch(Rest,"^(RE)",RegexOptions.IgnoreCase)){
				Results.Add (t_RE);
				ResultsV.Add(Regex.Match(Rest,"^(RE)",RegexOptions.IgnoreCase).Value);}

			if (Regex.IsMatch(Rest,"^(WH)",RegexOptions.IgnoreCase)){
				Results.Add (t_WH);
				ResultsV.Add(Regex.Match(Rest,"^(WH)",RegexOptions.IgnoreCase).Value);}

			if (Regex.IsMatch(Rest,"^(FARBE)",RegexOptions.IgnoreCase)){
				Results.Add (t_FARBE);
				ResultsV.Add(Regex.Match(Rest,"^(FARBE)",RegexOptions.IgnoreCase).Value);}

			if (Regex.IsMatch(Rest,"^(STIFT)",RegexOptions.IgnoreCase)){
				Results.Add (t_STIFT);
				ResultsV.Add(Regex.Match(Rest,"^(STIFT)",RegexOptions.IgnoreCase).Value);}

			if (Regex.IsMatch(Rest,"^(rot|blau|gruen|gelb|schwarz)",RegexOptions.IgnoreCase)){
				Results.Add (t_farbwert);
				ResultsV.Add(Regex.Match(Rest,"^(rot|blau|gruen|gelb|schwarz)",RegexOptions.IgnoreCase).Value);}

			if (Regex.IsMatch(Rest,"^([\\r\\s\\t\\n])",RegexOptions.IgnoreCase)){
				Results.Add (t_ignore);
				ResultsV.Add(Regex.Match(Rest,"^([\\r\\s\\t\\n])",RegexOptions.IgnoreCase).Value);}

			}catch{}
			int maxlength = 0;
			int besttoken = 0;
			AToken ret = new AToken();
		        ret.token = besttoken;
			for (int i = 0; i < Results.Count; i++){
				if (ResultsV[i].ToString().Length > maxlength)
				{
					maxlength = ResultsV[i].ToString().Length;
					besttoken = (int)Results[i];
		         	        ret.token = besttoken;
		                  	if (besttoken != 0)
			                ret.val   = ResultsV[i].ToString();
				}
			}
			return ret;
		}

	}
}
